/**
 * A stream of response messages. Messages can be read from
 * the stream via the AsyncIterable interface:
 *
 * ```typescript
 * for await (let message of response) {...
 * ```
 */
export interface RpcOutputStream<T extends object = object> extends AsyncIterable<T> {
    /**
     * Add a callback for every new datum.
     * If a new message arrived, the "message" argument is set.
     * If an error occurred, the "error" argument is set.
     * If no more messages will be arriving, the "done" argument is true.
     */
    onNext(callback: (message: T | undefined, error: Error | undefined, done: boolean) => void): RemoveListenerFn;
    /**
     * Add a callback for every new message.
     */
    onMessage(callback: (message: T) => void): RemoveListenerFn;
    /**
     * Add a callback for stream completion.
     * Called when all messages have been read without error.
     */
    onComplete(callback: () => void): RemoveListenerFn;
    /**
     * Add a callback for errors.
     * After an error occurred, the stream will no longer receive messages or
     * anything else.
     */
    onError(callback: (reason: Error) => void): RemoveListenerFn;
}
declare type NextCallback<T> = (message: T | undefined, error: Error | undefined, done: boolean) => void;
declare type RemoveListenerFn = () => void;
/**
 * A `RpcOutputStream` that you control.
 */
export declare class RpcOutputStreamController<T extends object = object> {
    /**
     * Is this stream already closed by a completion or error?
     */
    get closed(): boolean;
    private readonly _onNextListeners;
    private readonly _onMessageListeners;
    private readonly _onErrorListeners;
    private readonly _onCompleteListeners;
    private _pendingResult;
    private _closed;
    constructor();
    private assertOpen;
    onNext(callback: NextCallback<T>): RemoveListenerFn;
    onMessage(callback: (message: T) => void): RemoveListenerFn;
    onError(callback: (reason: Error) => void): RemoveListenerFn;
    onComplete(callback: () => void): RemoveListenerFn;
    [Symbol.asyncIterator](): AsyncIterator<T>;
    /**
     * Emits a new message. Throws if stream is already closed.
     */
    notifyMessage(message: T): void;
    /**
     * Closes the stream with an error. Throws if stream is already closed.
     */
    notifyError(error: Error): void;
    /**
     * Closes the stream successfully. Throws if stream is already closed.
     */
    notifyComplete(): void;
}
export {};
//# sourceMappingURL=rpc-output-stream.d.ts.map